# -*- coding: utf-8 -*-

"""Top-level package for spotware_connect."""

__author__ = """Marcus Santos"""
__email__ = 'marcus@marcus-santos.com'
__version__ = '0.1.1'

from .client import Client  # noqa
from .protobuf import Protobuf  # noqa
